@@ -84,38 +84,29 @@ function getReferrer(doc)
 function openNewWindowWith(url) 
 {
   urlSecurityCheck(url, document);
-  var newWin;
-  var wintype = document.firstChild.getAttribute('windowtype');
-  var referrer = getReferrer(document);
 
   // if and only if the current window is a browser window and it has a document with a character
   // set, then extract the current charset menu setting from the current document and use it to
   // initialize the new browser window...
-  if (window && (wintype == "navigator:browser") &&
-    window._content && window._content.document) {
-    var DocCharset = window._content.document.characterSet;
-    var charsetArg = "charset="+DocCharset;
+  var charsetArg = null;
+  var wintype = document.firstChild.getAttribute('windowtype');
+  if (wintype == "navigator:browser")
+    charsetArg = "charset=" + window._content.document.characterSet;
 
-    //we should "inherit" the charset menu setting in a new window
-    newWin = window.openDialog( getBrowserURL(), "_blank", "chrome,all,dialog=no", url, charsetArg, true, referrer );
-  }
-  else { // forget about the charset information.
-    newWin = window.openDialog( getBrowserURL(), "_blank", "chrome,all,dialog=no", url, null, true, referrer );
-  }
+  var referrer = getReferrer(document);
+  window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", url, charsetArg, referrer);
 }
 
 function openNewTabWith(url) 
 {
   urlSecurityCheck(url, document);
-  var wintype = document.firstChild.getAttribute('windowtype');
-  var referrer = getReferrer(document);
+  var browser = getBrowser();
 
-  if (window && (wintype == "navigator:browser")) {
-    var browser = getBrowser();
-    var t = browser.addTab(url, referrer); // open link in new tab
-    if (pref && !pref.getBoolPref("browser.tabs.loadInBackground"))
-      browser.selectedTab = t;
-  }
+  var referrer = getReferrer(document);
+  var tab = browser.addTab(url, referrer); // open link in new tab
+
+  if (pref && !pref.getBoolPref("browser.tabs.loadInBackground"))
+    browser.selectedTab = tab;
 }
 
 function findParentNode(node, parentNode)
@@ -417,6 +408,7 @@ nsHeaderSniffer.prototype = {
         this.contentType = channel.contentType;
         try {
           var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
+          var encodedChannel = channel.QueryInterface(Components.interfaces.nsIEncodedChannel);
           this.contentEncodingType = null;
           // There may be content-encodings on the channel.  Multiple content
           // encodings are allowed, eg "Content-Encoding: gzip, uuencode".  This
@@ -424,12 +416,12 @@ nsHeaderSniffer.prototype = {
           // uuencoded.  The encoding enumerator returns MIME types
           // corresponding to each encoding starting from the end, so the first
           // thing it returns corresponds to the outermost encoding.
-          var encodingEnumerator = httpChannel.contentEncodings;
+          var encodingEnumerator = encodedChannel.contentEncodings;
           if (encodingEnumerator && encodingEnumerator.hasMoreElements()) {
             try {
               this.contentEncodingType =
                 encodingEnumerator.getNext().
-                  QueryInterface(Components.interfaces.nsISupportsString).data;
+                  QueryInterface(Components.interfaces.nsISupportsCString).data;
             } catch (e) {
             }
           }
@@ -524,7 +516,7 @@ function appendFiltersForContentType(aFilePicker, aContentType, aSaveMode)
       aFilePicker.appendFilter(bundle.GetStringFromName("WebPageCompleteFilter"), "*.htm; *.html");
     aFilePicker.appendFilter(bundle.GetStringFromName("WebPageHTMLOnlyFilter"), "*.htm; *.html");
     if (aSaveMode == MODE_COMPLETE)
-      aFilePicker.appendFilter(bundle.GetStringFromName("TextOnlyFilter"), "*.txt");
+      aFilePicker.appendFilters(Components.interfaces.nsIFilePicker.filterText);
     break;
   default:
     var mimeInfo = getMIMEInfoForType(aContentType);
@@ -543,11 +535,11 @@ function appendFiltersForContentType(aFilePicker, aContentType, aSaveMode)
       if (extCount.value > 0) {
         aFilePicker.appendFilter(mimeInfo.Description, extString);
       } else {
-        aFilePicker.appendFilter(bundle.GetStringFromName("AllFilesFilter"), "*.*");
+        aFilePicker.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
       }        
     }
     else
-      aFilePicker.appendFilter(bundle.GetStringFromName("AllFilesFilter"), "*.*");
+      aFilePicker.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
     break;
   }
 } 
@@ -651,19 +643,63 @@ function getMIMEInfoForType(aMIMEType)
 function getDefaultFileName(aDefaultFileName, aNameFromHeaders, aDocumentURI, aDocument)
 {
   if (aNameFromHeaders)
-    return validateFileName(aNameFromHeaders);  // 1) Use the name suggested by the HTTP headers
+    // 1) Use the name suggested by the HTTP headers
+    return validateFileName(aNameFromHeaders);
 
-  var url = aDocumentURI.QueryInterface(Components.interfaces.nsIURL);
-  if (url.fileName != "")
-    return url.fileName;                        // 2) Use the actual file name, if present
+  try {
+    var url = aDocumentURI.QueryInterface(Components.interfaces.nsIURL);
+    if (url.fileName != "") {
+      // 2) Use the actual file name, if present
+      return unescape(url.fileName);
+    }
+  } catch (e) {
+    try {
+      // the file name might be non ASCII
+      // try unescape again with a characterSet
+      var textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
+                                   .getService(Components.interfaces.nsITextToSubURI);
+      var charset;
+      if (aDocument)
+        charset = aDocument.characterSet;
+      else if (document.commandDispatcher.focusedWindow)
+        charset = document.commandDispatcher.focusedWindow.document.characterSet;
+      else
+        charset = window._content.document.characterSet;
+      return textToSubURI.unEscapeURIForUI(charset, url.fileName);
+    } catch (e) {
+      // This is something like a wyciwyg:, data:, and so forth
+      // URI... no usable filename here.
+    }
+  }
   
-  if (aDocument && aDocument.title != "") 
-    return validateFileName(aDocument.title)    // 3) Use the document title
+  if (aDocument) {
+    var docTitle = validateFileName(aDocument.title).replace(/^\s+|\s+$/g, "");
 
+    if (docTitle != "") {
+      // 3) Use the document title
+      return docTitle;
+    }
+  }
+  
   if (aDefaultFileName)
-    return validateFileName(aDefaultFileName);  // 4) Use the caller-provided name, if any
+    // 4) Use the caller-provided name, if any
+    return validateFileName(aDefaultFileName);
 
-  return aDocumentURI.host;                     // 5) Use the host.
+  try {
+    if (aDocumentURI.host)
+      // 5) Use the host.
+      return aDocumentURI.host;
+  } catch (e) {
+    // Some files have no information at all, like Javascript generated pages
+  }
+  try {
+    // 6) Use the default file name
+    return getStringBundle().GetStringFromName("DefaultSaveFileName");
+  } catch (e) {
+    //in case localized string cannot be found
+  }
+  // 7) If all else fails, use "index"
+  return "index";
 }
 
 function validateFileName(aFileName)
@@ -702,7 +738,7 @@ function getNormalizedLeafName(aFile, aDefaultExtension)
 
 function getDefaultExtension(aFilename, aURI, aContentType)
 {
-  if (aContentType == "text/plain" || aContentType == "application/octet-stream" || aURI.scheme != "http" && aURI.scheme != "https")
+  if (aContentType == "text/plain" || aContentType == "application/octet-stream" || aURI.scheme == "ftp")
     return "";   // temporary fix for bug 120327
 
   // This mirrors some code in nsExternalHelperAppService::DoContent
