@@ -40,7 +40,7 @@
  *   on values set in the previous step.
  */
 
-
+var gSmtpHostNameIsIllegal = false;
 var accountArray;
 var gGenericAttributeTypes;
 var accounttree;
@@ -51,12 +51,14 @@ var currentPageId;
 var pendingServerId;
 var pendingPageId;
 var gPrefsBundle;
+var gBrandBundle;
 
 // services used
 var RDF;
 var accountManager;
 var smtpService;
 var nsPrefBranch;
+var gPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
 
 // widgets
 var duplicateButton;
@@ -94,6 +96,7 @@ function updateElementWithKeys(account, element, type) {
 // perform initialization here
 function onLoad() {
   gPrefsBundle = document.getElementById("bundle_prefs");
+  gBrandBundle = document.getElementById("bundle_brand");
 
   var selectedServer;
   var selectPage = null;
@@ -135,24 +138,28 @@ function sortAccountList(accounttree)
 
 function selectServer(server, selectPage)
 {
-  var selectedItem;
+  var selectedServer, selectedItem;
 
-  if (server) {
-    selectedItem = document.getElementById(server.serverURI);
-    if (selectedItem && selectPage) 
-      selectedItem = findSelectPage(selectedItem, selectPage);
-  }
+  if (server)
+    selectedServer = document.getElementById(server.serverURI);
+  if (!selectedServer)
+    selectedServer = getFirstAccount();
 
+  if (server && selectedServer && selectPage)
+    selectedItem = findSelectPage(selectedServer, selectPage);
   if (!selectedItem)
-    selectedItem = getFirstAccount();
+    selectedItem = selectedServer;
 
   var index = accounttree.contentView.getIndexOfItem(selectedItem);
   accounttree.treeBoxObject.selection.select(index);
 
-  var result = getServerIdAndPageIdFromTree(accounttree);
-  if (result) {
-    updateButtons(accounttree,result.serverId);
-  }
+  var lastItem = selectedServer.lastChild.lastChild;
+  if (lastItem.localName == "treeitem")
+    index = accounttree.contentView.getIndexOfItem(lastItem);
+
+  accounttree.treeBoxObject.ensureRowIsVisible(index);
+
+  updateButtons(accounttree, selectedServer.id);
 }
 
 function findSelectPage(selectServer, selectPage)
@@ -202,11 +209,44 @@ function findFirstTreeItem(tree) {
   return null;
 }
 
+function replaceWithDefaultSmtpServer(deletedSmtpServerKey)
+{
+  //First we replace the smtpserverkey in every identity
+
+  var allIdentities = accountManager.allIdentities;
+  var identitiesCount = allIdentities.Count();
+  for (var i = 0; i < identitiesCount; i++) {
+    var currentIdentity = allIdentities.QueryElementAt(i, Components.interfaces.nsIMsgIdentity);
+    if (currentIdentity.smtpServerKey == deletedSmtpServerKey)
+      currentIdentity.smtpServerKey = smtpService.defaultServer.key;
+  }
+
+  //When accounts have already been loaded in the panel then the first replacement will be 
+  //overwritten when the accountvalues are written out from the pagedata.
+  //we get the loaded accounts and check to make sure that the account exists for the accountid
+  //and that it has a default identity associated with it (to exclude smtpservers and local folders)
+  //Then we check only for the identity[type] and smtpServerKey[slot]
+  //and replace that with the default smtpserverkey if necessary.
+
+  for (var accountid in accountArray) {
+    var account = getAccountFromServerId(accountid);
+    if(account && account.defaultIdentity) {
+      var accountValues = accountArray[accountid];
+      if (accountValues['identity']['smtpServerKey'] == deletedSmtpServerKey)
+        setAccountValue(accountValues,'identity', 'smtpServerKey', smtpService.defaultServer.key);
+    }
+  }
+}
 function onAccept() {
   // Check if user/host have been modified.
   if (!checkUserServerChanges(true))
     return false;
 
+  if (gSmtpHostNameIsIllegal) {
+    gSmtpHostNameIsIllegal = false;
+    return false;
+  }
+
   onSave();
     // hack hack - save the prefs file NOW in case we crash
     try {
@@ -223,7 +263,27 @@ function onAccept() {
 // if so check if the new names already exists for an account.
 //
 function checkUserServerChanges(showAlert) {
+
+  if (smtpService.defaultServer) {
+    try {
+      var smtpHostName = top.frames["contentFrame"].document.getElementById("smtp.hostname");
+      if (hostnameIsIllegal(smtpHostName.value)) {
+        var alertTitle = gBrandBundle.getString("brandShortName");
+        var alertMsg = gPrefsBundle.getString("enterValidHostname");
+
+        if (gPromptService)
+          gPromptService.alert(window, alertTitle, alertMsg);
+        else
+          window.alert(alertMsg);
+       gSmtpHostNameIsIllegal = true;
+      }
+    }
+    catch (ex) {}
+  }
+
   var accountValues = getValueArrayFor(currentServerId);
+  if (!accountValues) 
+    return true;
   var pageElements = getPageFormElements();
 
   if (pageElements == null) return true;
@@ -234,27 +294,27 @@ function checkUserServerChanges(showAlert) {
   for (var i=0; i<pageElements.length; i++) {
     if (pageElements[i].id) {
       var vals = pageElements[i].id.split(".");
-      var type = vals[0];
-      var slot = vals[1];
-      //dump("In checkUserServerChanges() ***: accountValues[" + type + "][" + slot + "] = " + getFormElementValue(pageElements[i]) + "/" + accountValues[type][slot] + "\n");
-
-      // if this type doesn't exist (just removed) then return.
-      if (! accountValues[type]) return true;
-
-      if (slot == "realHostName") {
-        oldHost = accountValues[type][slot];
-        newHost = getFormElementValue(pageElements[i]);
-        hIndx = i;
+      if (vals.length >= 2) {
+          var type = vals[0];
+          var slot = vals[1];
+          //dump("In checkUserServerChanges() ***: accountValues[" + type + "][" + slot + "] = " + getFormElementValue(pageElements[i]) + "/" + accountValues[type][slot] + "\n");
+    
+          // if this type doesn't exist (just removed) then return.
+          if (!(type in accountValues) || !accountValues[type]) return true;
+    
+          if (slot == "realHostName") {
+            oldHost = accountValues[type][slot];
+            newHost = getFormElementValue(pageElements[i]);
+            hIndx = i;
+          }
+          else if (slot == "realUsername") {
+            oldUser = accountValues[type][slot];
+            newUser = getFormElementValue(pageElements[i]);
+            uIndx = i;
+          }
+          else if (slot == "type")
+            newType = getFormElementValue(pageElements[i]);
       }
-      else
-      if (slot == "realUsername") {
-        oldUser = accountValues[type][slot];
-        newUser = getFormElementValue(pageElements[i]);
-        uIndx = i;
-      }
-      else
-      if (slot == "type")
-        newType = getFormElementValue(pageElements[i]);
     }
   }
 
@@ -401,9 +461,9 @@ function onSetDefault(event) {
 }
 
 function onRemoveAccount(event) {
-    //dump("onRemoveAccount\n");
+  //dump("onRemoveAccount\n");
 
-    if (event.target.getAttribute("disabled") == "true") return;
+  if (event.target.getAttribute("disabled") == "true") return;
 
   var result = getServerIdAndPageIdFromTree(accounttree);
   if (!result) 
@@ -416,40 +476,40 @@ function onRemoveAccount(event) {
     selectServer(null, null); 
   }
 
-    var account = getAccountFromServerId(result.serverId);
-    if (!account) 
-      return;
+  var account = getAccountFromServerId(result.serverId);
+  if (!account) 
+    return;
 
-    var server = account.incomingServer;
-    var type = server.type;
+  var server = account.incomingServer;
+  var type = server.type;
 
-    var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type].getService(Components.interfaces.nsIMsgProtocolInfo);
-    var canDelete = protocolinfo.canDelete;
-    if (!canDelete) {
-        canDelete = server.canDelete;
-    }
-    if (!canDelete) 
-      return;
+  var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type].getService(Components.interfaces.nsIMsgProtocolInfo);
+  var canDelete = protocolinfo.canDelete;
+  if (!canDelete) {
+    canDelete = server.canDelete;
+  }
+  if (!canDelete) 
+    return;
 
-    var confirmRemoveAccount =
-      gPrefsBundle.getString("confirmRemoveAccount");
-    if (!window.confirm(confirmRemoveAccount)) 
-      return;
+  var confirmRemoveAccount =
+    gPrefsBundle.getString("confirmRemoveAccount");
+  if (!window.confirm(confirmRemoveAccount)) 
+    return;
 
-    try {
-      // clear cached data out of the account array
-      if (accountArray[result.serverId])
-        accountArray[result.serverId] = null;
-      currentServerId = currentPageId = null;
+  try {
+    // clear cached data out of the account array
+    if (accountArray[result.serverId])
+      accountArray[result.serverId] = null;
+    currentServerId = currentPageId = null;
 
-      accountManager.removeAccount(account);
-      selectServer(null, null);
-    }
-    catch (ex) {
-      dump("failure to remove account: " + ex + "\n");
-      var alertText = gPrefsBundle.getString("failedRemoveAccount");
-      window.alert(alertText);
-    }
+    accountManager.removeAccount(account);
+    selectServer(null, null);
+  }
+  catch (ex) {
+    dump("failure to remove account: " + ex + "\n");
+    var alertText = gPrefsBundle.getString("failedRemoveAccount");
+    window.alert(alertText);
+  }
 }
 
 function saveAccount(accountValues, account)
@@ -524,7 +584,7 @@ function saveAccount(accountValues, account)
          }
       }
       else {
-      if (dest[slot] != typeArray[slot]) {
+      if (slot in dest && dest[slot] != typeArray[slot]) {
         try {
           dest[slot] = typeArray[slot];
           } 
@@ -550,15 +610,15 @@ function updateButtons(tree,serverId) {
   //dump("account = " + account + "\n");
 
   if (account) {
-  var server = account.incomingServer;
-  var type = server.type;
+    var server = account.incomingServer;
+    var type = server.type;
 
     if (account.identities.Count() < 1)
       canSetDefault = false;
 
-  //dump("servertype = " + type + "\n");
+    //dump("servertype = " + type + "\n");
 
-  var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type].getService(Components.interfaces.nsIMsgProtocolInfo);
+    var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type].getService(Components.interfaces.nsIMsgProtocolInfo);
     canDuplicate = protocolinfo.canDuplicate;
     canDelete = protocolinfo.canDelete;
     if (!canDelete) {
@@ -569,6 +629,7 @@ function updateButtons(tree,serverId) {
     // HACK
     // if account is null, we have either selected a SMTP server, or there is a problem
     // either way, we don't want the user to be able to delete it or duplicate it
+
     canSetDefault = false;
     canDelete = (serverId == "http://home.netscape.com/NC-rdf#PageTitleFakeAccount");
     canDuplicate = false;
@@ -644,6 +705,11 @@ function showPage(serverId, pageId) {
   // check if user/host names have been changed
   checkUserServerChanges(false);
 
+  if (gSmtpHostNameIsIllegal) {
+    gSmtpHostNameIsIllegal = false;
+    return false;
+  }
+
   // save the previous page
   savePage(currentServerId);
 
@@ -696,36 +762,41 @@ function loadPage(pageId)
 //
 // save the values of the widgets to the given server
 //
-function savePage(serverId) {
-
-  if (!serverId) return;
+function savePage(serverId) 
+{
+  if (!serverId) 
+    return;
 
   // tell the page that it's about to save
-  if (top.frames["contentFrame"].onSave)
+  if ("onSave" in top.frames["contentFrame"])
       top.frames["contentFrame"].onSave();
 
   var accountValues = getValueArrayFor(serverId);
-  var pageElements = getPageFormElements();
+  if (!accountValues) 
+    return;
 
-  if (pageElements == null) return;
+  var pageElements = getPageFormElements();
+  if (!pageElements) 
+    return;
 
   // store the value in the account
   for (var i=0; i<pageElements.length; i++) {
       if (pageElements[i].id) {
         var vals = pageElements[i].id.split(".");
-        var type = vals[0];
-        var slot = vals[1];
-
-        setAccountValue(accountValues,
-                        type, slot,
-                        getFormElementValue(pageElements[i]));
+        if (vals.length >= 2) {
+            var type = vals[0];
+            var slot = vals[1];
+    
+            setAccountValue(accountValues,
+                            type, slot,
+                            getFormElementValue(pageElements[i]));
+         }
       }
   }
-
 }
 
 function setAccountValue(accountValues, type, slot, value) {
-  if (!accountValues[type])
+  if (!(type in accountValues))
     accountValues[type] = new Array;
 
   //dump("Form->Array: accountValues[" + type + "][" + slot + "] = " + value + "\n");
@@ -734,11 +805,11 @@ function setAccountValue(accountValues, type, slot, value) {
 }
 
 function getAccountValue(account, accountValues, type, slot, preftype, isGeneric) {
-  if (!accountValues[type])
+  if (!(type in accountValues))
     accountValues[type] = new Array;
 
   // fill in the slot from the account if necessary
-  if (accountValues[type][slot]== undefined) {
+  if (!(slot in accountValues[type]) || !accountValues[type][slot]) {
     // dump("Array->Form: lazily reading in the " + slot + " from the " + type + "\n");
     var server;
     if (account)
@@ -771,7 +842,7 @@ function getAccountValue(account, accountValues, type, slot, preftype, isGeneric
 
     if (source) {
       if (isGeneric) {
-        if (!gGenericAttributeTypes[type])
+        if (!(type in gGenericAttributeTypes))
           gGenericAttributeTypes[type] = new Array;
 
         // we need the preftype later, for setting when we save.
@@ -794,10 +865,15 @@ function getAccountValue(account, accountValues, type, slot, preftype, isGeneric
             break;
           }
       }
-      else {
-      accountValues[type][slot] = source[slot];
+      else if (slot in source) {
+        accountValues[type][slot] = source[slot];
+      } else {
+        accountValues[type][slot] = null;
+      }
+    }
+    else {
+      accountValues[type][slot] = null;
     }
-  }
   }
   var value = accountValues[type][slot];
   //dump("Array->Form: accountValues[" + type + "][" + slot + "] = " + value + "\n");
@@ -806,38 +882,46 @@ function getAccountValue(account, accountValues, type, slot, preftype, isGeneric
 //
 // restore the values of the widgets from the given server
 //
-function restorePage(pageId, serverId) {
-  if (!serverId) return;
-  var accountValues = getValueArrayFor(serverId);
-  var pageElements = getPageFormElements();
+function restorePage(pageId, serverId) 
+{
+  if (!serverId) 
+    return;
 
-  if (pageElements == null) return;
+  var accountValues = getValueArrayFor(serverId);
+  if (!accountValues) 
+    return;
+
+  var pageElements = getPageFormElements();
+  if (!pageElements) 
+    return;
 
   var account = getAccountFromServerId(serverId);
 
-  if (top.frames["contentFrame"].onPreInit)
+  if ("onPreInit" in top.frames["contentFrame"])
     top.frames["contentFrame"].onPreInit(account, accountValues);
 
   // restore the value from the account
   for (var i=0; i<pageElements.length; i++) {
       if (pageElements[i].id) {
         var vals = pageElements[i].id.split(".");
-        var type = vals[0];
-        var slot = vals[1];
-        // buttons are lockable, but don't have any data so we skip that part.
-        // elements that do have data, we get the values at poke them in.
-        if (pageElements[i].localName != "button") {
-          var value = getAccountValue(account, accountValues, type, slot, pageElements[i].getAttribute("preftype"), (pageElements[i].getAttribute("genericattr") == "true"));
-          setFormElementValue(pageElements[i], value);
-        }
-        updateElementWithKeys(account,pageElements[i],type);
-        var isLocked = getAccountValueIsLocked(pageElements[i]);
-        setEnabled(pageElements[i],!isLocked);
+        if (vals.length >= 2) {
+            var type = vals[0];
+            var slot = vals[1];
+            // buttons are lockable, but don't have any data so we skip that part.
+            // elements that do have data, we get the values at poke them in.
+            if (pageElements[i].localName != "button") {
+              var value = getAccountValue(account, accountValues, type, slot, pageElements[i].getAttribute("preftype"), (pageElements[i].getAttribute("genericattr") == "true"));
+              setFormElementValue(pageElements[i], value);
+            }
+            updateElementWithKeys(account,pageElements[i],type);
+            var isLocked = getAccountValueIsLocked(pageElements[i]);
+            setEnabled(pageElements[i],!isLocked);
+         }
       }
   }
 
   // tell the page that new values have been loaded
-  if (top.frames["contentFrame"].onInit)
+  if ("onInit" in top.frames["contentFrame"])
       top.frames["contentFrame"].onInit();
 
   // everything has succeeded, vervied by setting currentPageId
@@ -858,11 +942,9 @@ function getFormElementValue(formElement) {
     else
       return formElement.checked;
   }
-
   else if (type == "radiogroup" || type=="menulist") {
     return formElement.selectedItem.value;
   }
-
   else if (type == "textbox" &&
            formElement.getAttribute("datatype") == "nsIFileSpec") {
     if (formElement.value) {
@@ -885,16 +967,17 @@ function getFormElementValue(formElement) {
       return null;
     }
   }
-
   else if (type == "text") {
     var val = formElement.getAttribute("value");
     if (val) return val;
     else return null;
   }
-
-  else {
+  else if ("value" in formElement) {
     return formElement.value;
   }
+  else {
+    return null;
+  }
  }
  catch (ex) {
   dump("getFormElementValue failed, ex="+ex+"\n");
@@ -912,7 +995,7 @@ function setFormElementValue(formElement, value) {
   var type = formElement.localName;
   if (type == "checkbox") {
     if (value == undefined) {
-      if (formElement.defaultChecked)
+      if ("defaultChecked" in formElement && formElement.defaultChecked)
         formElement.checked = formElement.defaultChecked;
       else
         formElement.checked = false;
@@ -949,7 +1032,7 @@ function setFormElementValue(formElement, value) {
         dump("Still need to fix uninitialized filespec problem!\n");
       }
     } else {
-      if (formElement.defaultValue)
+      if ("defaultValue" in formElement)
         formElement.value = formElement.defaultValue;
       else
         formElement.value = "";
@@ -967,7 +1050,7 @@ function setFormElementValue(formElement, value) {
       }
 
     } else {
-      if (formElement.defaultValue)
+      if ("defaultValue" in formElement)
         formElement.value = formElement.defaultValue;
       else
         formElement.value = "";
@@ -984,7 +1067,7 @@ function setFormElementValue(formElement, value) {
   // let the form figure out what to do with it
   else {
     if (value == undefined) {
-      if (formElement.defaultValue)
+      if ("defaultValue" in formElement && formElement.defaultValue)
         formElement.value = formElement.defaultValue;
     }
     else
@@ -1019,9 +1102,11 @@ function getAccountFromServerId(serverId) {
 //
 function getPageFormElements() {
  try {
-  var pageElements =
-      top.frames["contentFrame"].document.getElementsByAttribute("wsm_persist", "true");
-  return pageElements;
+    if("getElementsByAttribute" in top.frames["contentFrame"].document) {
+        var pageElements =
+            top.frames["contentFrame"].document.getElementsByAttribute("wsm_persist", "true");
+        return pageElements;
+    }
  }
  catch (ex) {
   dump("getPageFormElements() failed: " + ex + "\n");
@@ -1035,7 +1120,7 @@ function getPageFormElements() {
 function getValueArrayFor(serverId) {
   if (serverId == undefined) serverId="global";
 
-  if (accountArray[serverId] == null) {
+  if (!(serverId in accountArray)) {
     accountArray[serverId] = new Array;
   }
 
