@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Blake Ross <blakeross@telocity.com>
  *   Peter Annema <disttsc@bart.nl>
+ *   Dean Tessman <dean_tessman@hotmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -51,7 +52,9 @@ var gBrandRegionBundle;
 var gLastValidURLStr = "";
 var gLastValidURL = null;
 var gHaveUpdatedToolbarState = false;
-var gClickSelectsAll = -1;
+var gClickSelectsAll = false;
+var gIgnoreFocus = false;
+var gIgnoreClick = false;
 
 var pref = null;
 
@@ -111,6 +114,21 @@ const gTabStripPrefListener =
   }
 };
 
+const gHomepagePrefListener =
+{
+  domain: "browser.startup.homepage",
+  observe: function(subject, topic, prefName)
+  {
+    // verify that we're changing the home page pref
+    if (topic != "nsPref:changed")
+      return;
+
+    var homePage = getHomePage();
+    if (homePage)
+      setTooltipText("home-button", homePage);
+  }
+};
+
 /**
 * Pref listener handler functions.
 * Both functions assume that observer.domain is set to 
@@ -159,7 +177,7 @@ function loadEventHandlers(event)
 function getContentAreaFrameCount()
 {
   var saveFrameItem = document.getElementById("savepage");
-  if (!_content.frames.length || !isDocumentFrame(document.commandDispatcher.focusedWindow))
+  if (!_content || !_content.frames.length || !isDocumentFrame(document.commandDispatcher.focusedWindow))
     saveFrameItem.setAttribute("hidden", "true");
   else
     saveFrameItem.removeAttribute("hidden");
@@ -358,6 +376,7 @@ function Startup()
 
   addPrefListener(gButtonPrefListener); 
   addPrefListener(gTabStripPrefListener);
+  addPrefListener(gHomepagePrefListener);
 
   window.browserContentListener =
     new nsBrowserContentListener(window, getBrowser());
@@ -434,8 +453,8 @@ function Startup()
     
     if (uriToLoad && uriToLoad != "about:blank") {
       gURLBar.value = uriToLoad;
-      if ("arguments" in window && window.arguments.length >= 4) {
-        loadURI(uriToLoad, window.arguments[3]);
+      if ("arguments" in window && window.arguments.length >= 3) {
+        loadURI(uriToLoad, window.arguments[2]);
       } else {
         loadURI(uriToLoad);
       }
@@ -452,9 +471,9 @@ function Startup()
         return;
     }
 
-    const navBar = document.getElementById("nav-bar");
     // Focus the content area unless we're loading a blank page
-    if ((!uriToLoad || uriToLoad == "about:blank") && !navBar.hidden && window.locationbar.visible)
+    var navBar = document.getElementById("nav-bar");
+    if (uriToLoad == "about:blank" && !navBar.hidden && window.locationbar.visible)
       setTimeout(WindowFocusTimerCallback, 0, gURLBar);
     else
       setTimeout(WindowFocusTimerCallback, 0, _content);
@@ -477,9 +496,11 @@ function Startup()
   // initiated by a web page script
   addEventListener("fullscreen", onFullScreen, false);
 
+  // does clicking on the urlbar select its contents?
+  gClickSelectsAll = pref.getBoolPref("browser.urlbar.clickSelectsAll");
+
   // now load bookmarks after a delay
   setTimeout(LoadBookmarksCallback, 0);
-
 }
 
 function LoadBookmarksCallback()
@@ -490,7 +511,7 @@ function LoadBookmarksCallback()
                                     .getService(Components.interfaces.nsIBookmarksService);
     gBookmarksService.ReadBookmarks();
     // tickle personal toolbar to load personal toolbar items
-    var personalToolbar = document.getElementById("innermostBox");
+    var personalToolbar = document.getElementById("NC:PersonalToolbarFolder");
     personalToolbar.builder.rebuild();
   } catch (e) {
   }
@@ -553,6 +574,7 @@ function Shutdown()
   // unregister us as a pref listener
   removePrefListener(gButtonPrefListener);
   removePrefListener(gTabStripPrefListener);
+  removePrefListener(gHomepagePrefListener);
 
   window.browserContentListener.close();
   // Close the app core.
@@ -655,7 +677,7 @@ function OpenBookmarkGroup(element, datasource)
   var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                           .getService(Components.interfaces.nsIRDFService);
   var resource = rdf.GetResource(id, true);
-  return OpenBookmarkGroupFromResource(resource, datasource, rdf);
+  OpenBookmarkGroupFromResource(resource, datasource, rdf);
 }
 
 function OpenBookmarkGroupFromResource(resource, datasource, rdf) {
@@ -667,8 +689,8 @@ function OpenBookmarkGroupFromResource(resource, datasource, rdf) {
   var tabCount = tabPanels.length;
   var index = 0;
   for (; containerChildren.hasMoreElements(); ++index) {
-    var resource = containerChildren.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
-    var target = datasource.GetTarget(resource, urlResource, true);
+    var res = containerChildren.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
+    var target = datasource.GetTarget(res, urlResource, true);
     if (target) {
       var uri = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
       gBrowser.addTab(uri);
@@ -678,9 +700,11 @@ function OpenBookmarkGroupFromResource(resource, datasource, rdf) {
   if (index == 0)
     return; // If the bookmark group was completely invalid, just bail.
      
-  // Select the first tab in the group.
-  var tabs = gBrowser.mTabContainer.childNodes;
-  gBrowser.selectedTab = tabs[tabCount];
+  // Select the first tab in the group if we aren't loading in the background.
+  if (!pref.getBoolPref("browser.tabs.loadInBackground")) {
+    var tabs = gBrowser.mTabContainer.childNodes;
+    gBrowser.selectedTab = tabs[tabCount];
+  }
 }
 
 function OpenBookmarkURL(node, datasources)
@@ -774,7 +798,8 @@ function ensureSearchPref()
   var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
   var ds = rdf.GetDataSource("rdf:internetsearch");
   var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
-  kNC_Name = rdf.GetResource("http://home.netscape.com/NC-rdf#Name");
+  var kNC_Name = rdf.GetResource("http://home.netscape.com/NC-rdf#Name");
+  var defaultEngine;
   try {
     defaultEngine = mPrefs.getCharPref("browser.search.defaultengine");
   } catch(ex) {
@@ -833,7 +858,7 @@ function OpenSearch(tabName, forceDialogFlag, searchStr, newWindowFlag)
     defaultSearchURL = fallbackDefaultSearchURL;
 
   if (!searchStr) {
-    loadURI(gNavigatorRegionBundle.getString("otherSearchURL"));
+    BrowserSearchInternet();
   } else {
 
     //Check to see if location bar field is a url
@@ -850,7 +875,7 @@ function OpenSearch(tabName, forceDialogFlag, searchStr, newWindowFlag)
 
       if (forceDialogFlag || searchMode == 1) {
         // Use a single search dialog
-        var windowManager = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"]
+        var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                       .getService(Components.interfaces.nsIWindowMediator);
 
         var searchWindow = windowManager.getMostRecentWindow("search:window");
@@ -915,6 +940,37 @@ function isSearchPanelOpen()
          );
 }
 
+function BrowserSearchInternet()
+{
+  try {
+    var searchEngineURI = pref.getCharPref("browser.search.defaultengine");
+    if (searchEngineURI) {          
+      var searchRoot = getSearchUrl("searchForm");
+      if (searchRoot) {
+        loadURI(searchRoot);
+        return;
+      } else {
+        // Get a search URL and guess that the front page of the site has a search form.
+        var searchDS = Components.classes["@mozilla.org/rdf/datasource;1?name=internetsearch"]
+                                 .getService(Components.interfaces.nsIInternetSearchService);
+        searchURL = searchDS.GetInternetSearchURL(searchEngineURI, "ABC", 0, 0, {value:0});
+        if (searchURL) {
+          searchRoot = searchURL.match(/[a-z]+:\/\/[a-z.-]+/);
+          if (searchRoot) {
+            loadURI(searchRoot + "/");
+            return;
+          }
+        }
+      }
+    }
+  } catch (ex) {
+  }
+
+  // Fallback if the stuff above fails: use the hard-coded search engine
+  loadURI(gNavigatorRegionBundle.getString("otherSearchURL"));
+}
+
+
 //Note: BrowserNewEditorWindow() was moved to globalOverlay.xul and renamed to NewEditorWindow()
 
 function BrowserOpenWindow()
@@ -974,7 +1030,7 @@ function enableBookmarks()
 function BrowserEditBookmarks()
 {
   // Use a single sidebar bookmarks dialog
-  var windowManager = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"]
+  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                 .getService(Components.interfaces.nsIWindowMediator);
 
   var bookmarksWindow = windowManager.getMostRecentWindow("bookmarks:manager");
@@ -1043,16 +1099,83 @@ function BrowserLoadURL(aTriggeringEvent)
   if (url.match(/^view-source:/)) {
     BrowserViewSourceOfURL(url.replace(/^view-source:/, ""), null, null);
   } else {
-    if (pref && pref.getBoolPref("browser.tabs.opentabfor.urlbar") &&
-        getBrowser().localName == "tabbrowser" &&
-        aTriggeringEvent && 'ctrlKey' in aTriggeringEvent &&
-        aTriggeringEvent.ctrlKey) {
-      var t = getBrowser().addTab(getShortcutOrURI(url)); // open link in new tab
-      getBrowser().selectedTab = t;
+    // Check the pressed modifiers: (also see bug 97123)
+    // Modifier Mac | Modifier PC | Action
+    // -------------+-------------+-----------
+    // Command      | Control     | New Window/Tab
+    // Shift+Cmd    | Shift+Ctrl  | New Window/Tab behind current one
+    // Option       | Shift       | Save URL (show Filepicker)
+
+    // If false, the save modifier is Alt, which is Option on Mac.
+    var modifierIsShift = true;
+    try {
+      modifierIsShift = pref.getBoolPref("ui.key.saveLink.shift");
+    }
+    catch (ex) {}
+
+    var shiftPressed = false;
+    var saveModifier = false; // if the save modifier was pressed
+    if (aTriggeringEvent && 'shiftKey' in aTriggeringEvent &&
+        'altKey' in aTriggeringEvent) {
+      saveModifier = modifierIsShift ? aTriggeringEvent.shiftKey
+                     : aTriggeringEvent.altKey;
+      shiftPressed = aTriggeringEvent.shiftKey;
+    }
+
+    url = getShortcutOrURI(url);
+    // Accept both Control and Meta (=Command) as New-Window-Modifiers
+    if (aTriggeringEvent &&
+        (('ctrlKey' in aTriggeringEvent && aTriggeringEvent.ctrlKey) ||
+         ('metaKey' in aTriggeringEvent && aTriggeringEvent.metaKey))) {
+      // Check if user requests Tabs instead of windows
+      var openTab = false;
+      try {
+        openTab = pref.getBoolPref("browser.tabs.opentabfor.urlbar");
+      }
+      catch (ex) {}
+
+      if (openTab && getBrowser().localName == "tabbrowser") {
+        // Open link in new tab
+        var t = getBrowser().addTab(url);
+        // Focus new tab unless shift is pressed
+        if (!shiftPressed)
+          getBrowser().selectedTab = t;
+      }
+      else {
+        // Open a new window with the URL
+        var newWin = openDialog(getBrowserURL(), "_blank", "all,dialog=no", url);
+        // Reset url in the urlbar, copied from handleURLBarRevert()
+        var oldURL = getWebNavigation().currentURI.spec;
+        if (oldURL != "about:blank") {
+          gURLBar.value = oldURL;
+          SetPageProxyState("valid", null);
+        }
+        else
+          gURLBar.value = "";
+
+        // Focus old window if shift was pressed, as there's no
+        // way to open a new window in the background
+        // XXX this doesn't seem to work
+        if (shiftPressed) {
+          //newWin.blur();
+          content.focus();
+        }
+      }
+    }
+    else if (saveModifier) {
+      // Firstly, fixup the url so that (e.g.) "www.foo.com" works
+      const nsIURIFixup = Components.interfaces.nsIURIFixup;
+      var uriFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(nsIURIFixup);
+      url = uriFixup.createFixupURI(url, nsIURIFixup.FIXUP_FLAGS_MAKE_ALTERNATE_URI).spec;
+      // Open filepicker to save the url
+      saveURL(url, "");
+    }
+    else {
+      // No modifier was pressed, load the URL normally and
+      // focus the content area
+      loadURI(url);
+      content.focus();
     }
-    else  
-      loadURI(getShortcutOrURI(url));
-    _content.focus();
   }
 }
 
@@ -1112,7 +1235,7 @@ function readFromClipboard()
     trans.getTransferData("text/unicode", data, dataLen);
 
     if (data) {
-      data = data.value.QueryInterface(Components.interfaces.nsISupportsWString);
+      data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
       url = data.data.substring(0, dataLen.value / 2);
     }
   } catch (ex) {
@@ -1447,12 +1570,13 @@ function stylesheetSwitchAll(frameset, title) {
 
 function applyTheme(themeName)
 {
-  var name = themeName.getAttribute("name");
+  var id = themeName.getAttribute('id'); 
+  var name=id.substring('urn:mozilla.skin.'.length, id.length);
   if (!name)
     return;
 
   var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
-    .getService(Components.interfaces.nsIChromeRegistry);
+    .getService(Components.interfaces.nsIXULChromeRegistry);
 
   var oldTheme = false;
   try {
@@ -1461,8 +1585,8 @@ function applyTheme(themeName)
   catch(e) {
   }
 
-  var str = Components.classes["@mozilla.org/supports-wstring;1"]
-                      .createInstance(Components.interfaces.nsISupportsWString);
+  var str = Components.classes["@mozilla.org/supports-string;1"]
+                      .createInstance(Components.interfaces.nsISupportsString);
 
   var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
   if (oldTheme) {
@@ -1480,7 +1604,7 @@ function applyTheme(themeName)
 
       str.data = true;
       pref.setComplexValue("general.skins.removelist." + name,
-                           Components.interfaces.nsISupportsWString, str);
+                           Components.interfaces.nsISupportsString, str);
       
       if (inUse)
         chromeRegistry.refreshSkins();
@@ -1493,9 +1617,9 @@ function applyTheme(themeName)
  // we STILL haven't fixed editor skin switch problems                         
  // hacking around it yet again                                                
 
- str.data = themeName.getAttribute("name");
- pref.setComplexValue("general.skins.selectedSkin", Components.interfaces.nsISupportsWString, str);
-
+ str.data = name;
+ pref.setComplexValue("general.skins.selectedSkin", Components.interfaces.nsISupportsString, str);
+ 
  // shut down quicklaunch so the next launch will have the new skin
  var appShell = Components.classes['@mozilla.org/appshell/appShellService;1'].getService();
  appShell = appShell.QueryInterface(Components.interfaces.nsIAppShellService);
@@ -1503,8 +1627,8 @@ function applyTheme(themeName)
    appShell.nativeAppSupport.isServerMode = false;
  }
  catch(ex) {
- } 
-
+ }
+ 
  if (promptService) {                                                          
    var dialogTitle = gNavigatorBundle.getString("switchskinstitle");           
    var brandName = gBrandBundle.getString("brandShortName");                   
@@ -1520,26 +1644,27 @@ function getNewThemes()
 
 function URLBarFocusHandler(aEvent)
 {
-  if (gURLBar) {
-    if (gClickSelectsAll == -1)
-      gClickSelectsAll = pref.getBoolPref("browser.urlbar.clickSelectsAll");
-    if (gClickSelectsAll)
-      gURLBar.setSelectionRange(0, gURLBar.textLength);
-  }
+  if (gIgnoreFocus)
+    gIgnoreFocus = false;
+  else if (gClickSelectsAll)
+    gURLBar.select();
 }
 
-function URLBarBlurHandler(aEvent)
+function URLBarMouseDownHandler(aEvent)
 {
-  // XXX why the hell do we have to do this?
-  if (gClickSelectsAll)
+  if (gURLBar.hasAttribute("focused")) {
+    gIgnoreClick = true;
+  } else {
+    gIgnoreFocus = true;
+    gIgnoreClick = false;
     gURLBar.setSelectionRange(0, 0);
+  }
 }
 
-function URLBarKeyupHandler(aEvent)
+function URLBarClickHandler(aEvent)
 {
-  if (aEvent.keyCode == aEvent.DOM_VK_TAB ) {
-    ShowAndSelectContentsOfURLBar();
-  }
+  if (!gIgnoreClick && gClickSelectsAll && gURLBar.selectionStart == gURLBar.selectionEnd)
+    gURLBar.select();
 }
 
 // This function gets the "windows hooks" service and has it check its setting
@@ -1776,7 +1901,7 @@ function toHistory()
 {
   // Use a single sidebar history dialog
 
-  var cwindowManager = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService();
+  var cwindowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
   var iwindowManager = Components.interfaces.nsIWindowMediator;
   var windowManager  = cwindowManager.QueryInterface(iwindowManager);
 
@@ -1801,4 +1926,22 @@ function toHistory()
 
 }
 
-
+function checkTheme()
+{
+  var theSkinKids = document.getElementById("theme");
+  var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
+    .getService(Components.interfaces.nsIXULChromeRegistry);
+  for (var i = 0; i < theSkinKids.childNodes.length; ++i) {
+    var child = theSkinKids.childNodes[i];
+    var id=child.getAttribute("id");
+    if (id.length > 0) {
+      var themeName = id.substring('urn:mozilla:skin:'.length, id.length);       
+      var selected = chromeRegistry.isSkinSelected(themeName, true);
+      if (selected == Components.interfaces.nsIChromeRegistry.FULL) {
+        var menuitem=document.getElementById(id);
+        menuitem.setAttribute("checked", true);
+        break;
+      }
+    }
+  } 
+}
